@@ -1,3 +1,25 @@
+# $Id: SympaTransport.pm 10083 2014-01-01 00:12:12Z sikeda $
+#
+# Sympa - SYsteme de Multi-Postage Automatique
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 package SOAP::Transport::HTTP::FCGI::Sympa;
 
 use strict;
@@ -7,8 +29,6 @@ use SympaSession;
 use SOAP::Transport::HTTP;
 @ISA = qw(SOAP::Transport::HTTP::FCGI);
 
-1;
-
 sub request {
     my $self = shift;
     
@@ -54,11 +74,14 @@ sub response {
     
     if (my $response = $_[0]) {
 	if (defined $ENV{'SESSION_ID'}) {
-	    my $expire = $main::param->{'user'}{'cookie_delay'} || $main::wwsconf->{'cookie_expire'};
-	    my $cookie = &cookielib::set_cookie_soap($ENV{'SESSION_ID'}, $ENV{'SERVER_NAME'}, $expire);
-	
+	    my $expire =
+		$main::param->{'user'}{'cookie_delay'} ||
+		$main::wwsconf->{'cookie_expire'};
+	    my $cookie = SympaSession::soap_cookie2(
+		$ENV{'SESSION_ID'}, $ENV{'SERVER_NAME'}, $expire
+	    );
 	    $response->headers->push_header('Set-Cookie2' => $cookie);
-	  }
+	}
     }
     
     $self->SUPER::request(@_);
@@ -85,3 +108,5 @@ sub handle ($$) {
     }
     return undef;
 }
+
+1;
