@@ -1,3 +1,12 @@
+# $Id: sympasoap.pm 11563 2014-10-17 07:17:36Z sikeda $
+
+# Sympa - SYsteme de Multi-Postage Automatique
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -9,8 +18,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 package SOAP::XMLSchema1999::Serializer;
 
@@ -175,7 +183,8 @@ sub login {
     } 
 
     ## Create SympaSession object
-    my $session = new SympaSession($robot, {'cookie' => $ENV{'SESSION_ID'}});
+    my $session = new SympaSession($robot,
+	{ 'cookie' => SympaSession::encrypt_session_id($ENV{'SESSION_ID'}) });
     $ENV{'USER_EMAIL'} = $email;
     $session->{'email'} = $email;
     $session->store();
@@ -184,7 +193,8 @@ sub login {
     $ENV{'SESSION_ID'} = $session->{'id_session'};
 
     ## Also return the cookie value
-    return SOAP::Data->name('result')->type('string')->value($ENV{'SESSION_ID'});
+    return SOAP::Data->name('result')->type('string')
+	->value(SympaSession::encrypt_session_id($ENV{'SESSION_ID'}));
 }
 
 sub casLogin {
@@ -250,7 +260,8 @@ sub casLogin {
     }
 
     ## Create SympaSession object
-    my $session = new SympaSession($robot, {'cookie' => $ENV{'SESSION_ID'}});
+    my $session = new SympaSession($robot,
+	{ 'cookie' => SympaSession::encrypt_session_id($ENV{'SESSION_ID'}) });
     $ENV{'USER_EMAIL'} = $email;
     $session->{'email'} = $email;
     $session->store();
@@ -259,18 +270,19 @@ sub casLogin {
     $ENV{'SESSION_ID'} = $session->{'id_session'};
 
     ## Also return the cookie value
-    return SOAP::Data->name('result')->type('string')->value($ENV{'SESSION_ID'});
+    return SOAP::Data->name('result')->type('string')
+	->value(SympaSession::encrypt_session_id($ENV{'SESSION_ID'}));
 }
 
 ## Used to call a service as an authenticated user without using HTTP cookies
 ## First parameter is the secret contained in the cookie
 sub authenticateAndRun {
     my ($self, $email, $cookie, $service, $parameters) = @_;
+    my $session_id;
 
-    my $session_id = $cookie;
-    &do_log('notice','authenticateAndRun(%s,%s,%s,%s)', $email, $session_id, $service, join(',',@$parameters));
+    &do_log('notice','authenticateAndRun(%s,%s,%s,%s)', $email, $cookie, $service, join(',',@$parameters));
 
-    unless ($session_id and $service) {
+    unless ($cookie and $service) {
       &do_log('err', "Missing parameter");
 	die SOAP::Fault->faultcode('Client')
 	    ->faultstring('Incorrect number of parameters')
@@ -280,8 +292,11 @@ sub authenticateAndRun {
     
 
     ## Provided email is not trusted, we fetch the user email from the session_table instead
-    my $session = new SympaSession($ENV{'SYMPA_ROBOT'},{'cookie' => $session_id});
-    $email = $session->{'email'} if (defined $session);
+    my $session = new SympaSession($ENV{'SYMPA_ROBOT'},{'cookie' => $cookie});
+    if (defined $session) {
+	$email = $session->{'email'};
+	$session_id = $session->{'id_session'};
+    }
     unless ($email or ($email eq 'unkown')  ) {
       &do_log('err', "Failed to authenticate user with session ID $session_id");
       die SOAP::Fault->faultcode('Client')
@@ -547,12 +562,19 @@ sub createList {
 	    ->faultdetail("Missing required parameter(s) : $reject");	
     }
     # check authorization
-    my $result = &Scenario::request_action('create_list','md5',$robot,
-					 {'sender' => $sender,
-					  'remote_host' => $ENV{'REMOTE_HOST'},
-					  'remote_addr' =>  $ENV{'REMOTE_ADDR'},
-					  'remote_application_name' =>  $ENV{'remote_application_name'} }
-					 );
+    my $result = &Scenario::request_action('create_list', 'md5', $robot,
+		{
+			'sender' => $sender,
+			'candidate_listname' => $listname,
+			'candidate_subject' => $subject,
+			'candidate_template' => $template,
+			'candidate_info' => $description,
+			'candidate_topics' => $topics,
+			'remote_host' => $ENV{'REMOTE_HOST'},
+			'remote_addr' =>  $ENV{'REMOTE_ADDR'},
+			'remote_application_name' =>  $ENV{'remote_application_name'}
+		}
+	);
     my $r_action;
     my $reason;
     if (ref($result) eq 'HASH') {
@@ -697,6 +719,12 @@ sub add {
 	    ->faultstring('Incorrect number of parameters')
 	    ->faultdetail('Use : <email>');
     }
+    unless (tools::valid_email($email)) {
+        my $error = "Invalid email address provided: '$email'";
+        die SOAP::Fault->faultcode('Client')
+            ->faultstring('Unable to add user')
+            ->faultdetail($error);
+    }
     my $list = new List ($listname, $robot);
     unless ($list) {
 	&Log::do_log('info', 'add %s@%s %s from %s refused, no such list ', $listname,$robot,$email,$sender);
@@ -753,7 +781,22 @@ sub add {
 	$u->{'email'} = $email;
 	$u->{'gecos'} = $gecos || $u2->{'gecos'};
 	$u->{'date'} = $u->{'update_date'} = time;
-	$u->{'password'} = $u2->{'password'} || &tools::tmp_passwd($email) ;
+
+	# If Password validation is enabled check the submitted password 
+  # against the site configured constraints
+	if ($u2->{'password'}) {
+		if (my $result = &tools::password_validation($u->{'password'})) {
+			&Log::do_log('info', 'add %s@%s %s from %s : scenario error', $listname,$robot,$email,$sender);
+			die SOAP::Fault->faultcode('Server')
+		    ->faultstring('Weak password')
+				->faultdetail('Weak password: '. $result);
+		}
+		$u->{'password'} = $u2->{'password'};
+	}
+	else {
+		$u->{'password'} = &tools::tmp_passwd($email) ;
+	}
+
 	$u->{'lang'} = $u2->{'lang'} || $list->{'admin'}{'lang'};
 	
 	unless ($list->add_user($u)) {
@@ -1379,14 +1422,9 @@ sub struct_to_soap {
 	my $formated_data;
 	foreach my $k (keys %$data) {
 	    my $one_data = $k.'='.$data->{$k};
-	    
-	    ## Decode from the current charset to perl internal charset
-	    ## Then encode strings to UTF-8
 	    if (require "Encode.pm") {
-		# $one_data = &Encode::decode(&Language::GetCharset(), $one_data);
-		$one_data = &Encode::encode('utf-8', $one_data);
+		$one_data = &Encode::decode('utf-8', $one_data);
 	    }
-
 	    push @all, $one_data;
 	}
 
