@@ -1,9 +1,12 @@
 /* boucequeue.c - This program does the non-delivery deports spooling
-  RCS Identication ; $Revision: 6491 $ ; $Date: 2010-06-09 11:50:02 +0200 (mer 09 jun 2010) $ 
+  RCS Identication ; $Revision: 10083 $ ; $Date: 2014-01-01 01:12:12 +0100 (mer. 01 janv. 2014) $ 
 
   Sympa - SYsteme de Multi-Postage Automatique
-  Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-  Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+
+  Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+  Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+  2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+  Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -16,8 +19,8 @@
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -25,6 +28,10 @@
 #include <sysexits.h>
 #include <string.h>
 #include <stdlib.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <time.h>
 
 static char     qfile[128];
 static char     buf[16384];
