@@ -1,9 +1,12 @@
 # Command.pm - this module does the mail commands processing
-# RCS Identication ; $Revision: 7327 $ ; $Date: 2012-04-17 05:59:10 +0200 (mar 17 avr 2012) $ 
+# RCS Identication ; $Revision: 10127 $ ; $Date: 2014-01-05 16:07:59 +0100 (dim. 05 janv. 2014) $ 
 
 # Sympa - SYsteme de Multi-Postage Automatique
-# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-# Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,8 +19,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 package Commands;
 
@@ -179,7 +181,7 @@ sub help {
 
     &do_log('debug', 'Commands::help to robot %s',$robot);
 
-    # sa ne prends pas en compte la structure des rpertoires par lang.
+    # sa ne prends pas en compte la structure des répertoires par lang.
     # we should make this utilize Template's chain of responsibility
     if ((-r "$etc/mail_tt2/helpfile.tt2")||("$etc/$robot/mail_tt2/helpfile.tt2")) {
   
@@ -361,7 +363,7 @@ sub stats {
 		     );
 	
 	unless ($list->send_file('stats_report', $sender, $robot, {'stats' => \%stats, 
-								   'subject' => "STATS $list->{'name'}",
+								   'subject' => "STATS $list->{'name'}", # compat <= 6.1.17.
 								   'auto_submitted' => 'auto-replied'})) {
 	    &do_log('notice',"Unable to send template 'stats_reports' to $sender");
 	    &report::reject_report_cmd('intern_quiet','',{'listname'=> $l},$cmd_line,$sender,$robot);
@@ -637,7 +639,7 @@ sub review {
 	} while ($user = $list->get_next_user());
 	unless ($list->send_file('review', $sender, $robot, {'users' => \@users, 
 							     'total' => $list->get_total(),
-							     'subject' => "REVIEW $listname",
+							     'subject' => "REVIEW $listname", # Compat <= 6.1.17.
 							     'auto_submitted' => 'auto-replied'})) {
 	    &do_log('notice',"Unable to send template 'review' to $sender");
 	    &report::reject_report_cmd('intern_quiet','',{'listname'=>$listname},$cmd_line,$sender,$robot);
@@ -965,7 +967,11 @@ sub info {
 	    $data->{'digest'} = join (',', @days).' '.$list->{'admin'}{'digest'}{'hour'}.':'.$list->{'admin'}{'digest'}{'minute'};
 	}
 
-	$data->{'available_reception_mode'} = $list->available_reception_mode();
+	## Reception mode
+	$data->{'available_reception_mode'} =
+	    $list->available_reception_mode();
+	$data->{'available_reception_modeA'} =
+	    [$list->available_reception_mode()];
 
 	my $wwsympa_url = &Conf::get_robot_conf($robot, 'wwsympa_url');
 	$data->{'url'} = $wwsympa_url.'/info/'.$list->{'name'};
@@ -1936,7 +1942,16 @@ sub set {
     if ($mode =~ /^(mail|nomail|digest|digestplain|summary|notice|txt|html|urlize|not_me)/){
         # Verify that the mode is allowed
         if (! $list->is_available_reception_mode($mode)) {
-	    &report::reject_report_cmd('user','available_reception_mode',{'listname' => $which, 'modes' => $list->available_reception_mode},$cmd_line); 
+	    &report::reject_report_cmd(
+		'user',
+		'available_reception_mode',
+		{   'listname'        => $which,
+		    'modes'           =>
+			join(' ', $list->available_reception_mode()),
+		    'reception_modes' => [$list->available_reception_mode()]
+		},
+		$cmd_line
+	    ); 
 	    &do_log('info','SET %s %s from %s refused, mode not available', $which, $mode, $sender);
 	    return 'not_allowed';
 	}
@@ -2063,7 +2078,7 @@ sub distribute {
 	unless ($numsmtp) {
 	    &do_log('info', 'Message for %s from %s accepted but all subscribers use digest,nomail or summary',$which, $sender);
 	} 
-	&do_log('info', 'Message for %s from %s accepted (%d seconds, %d sessions, %d subscribers), message-id=%s, size=%d', $which, $sender, time - $start_time, $numsmtp, $list->get_total(), $hdr->get('Message-Id'), $bytes);
+	&do_log('info', 'Message for %s from %s accepted (%d seconds, %d sessions, %d subscribers), message-id=%s, size=%d', $which, $sender, time - $start_time, $numsmtp, $list->get_total(), $msg_id, $bytes);
 
 	unless ($quiet) {
 	    unless (&report::notice_report_msg('message_distributed',$sender,{'key' => $key,'message' => $message},$robot,$list)) {
@@ -2083,7 +2098,7 @@ sub distribute {
 	unless ($quiet) {
 	    &report::notice_report_msg('message_in_distribution_spool',$sender,{'key' => $key,'message' => $message},$robot,$list);
 	}
-	&do_log('info', 'Message for %s from %s moved in spool %s for distribution message-id=%s', $name, $sender, $Conf{'queuedistribute'},$hdr->get('Message-Id'));
+	&do_log('info', 'Message for %s from %s moved in spool %s for distribution message-id=%s', $name, $sender, $Conf{'queuedistribute'}, $msg_id);
     }
     unlink($file);
     
@@ -2264,7 +2279,7 @@ sub confirm {
 		&report::notice_report_msg('message_confirmed_and_in_distribution_spool',$sender,{'key' => $key,'message' => $message},$robot,$list);
 	    }
 
-	    &do_log('info', 'Message for list %s from %s confirmed ; file %s moved to spool %s for distribution message-id=%s', $name, $sender, $file, $Conf{'queuedistribute'},$hdr->get('Message-Id'));
+	    &do_log('info', 'Message for list %s from %s confirmed ; file %s moved to spool %s for distribution message-id=%s', $name, $sender, $file, $Conf{'queuedistribute'}, $msgid);
 	}
 	unlink($file);
 	
@@ -2355,8 +2370,7 @@ sub reject {
     unless  ($#sender_hdr == -1) {
 	my $rejected_sender = $sender_hdr[0]->address;
 	my %context;
-	$context{'subject'} = &MIME::EncWords::decode_mimewords($message->head->get('subject'), Charset=>'utf8');
-	chomp($context{'subject'});
+	$context{'subject'} = &tools::decode_header($message, 'Subject');
 	$context{'rejected_by'} = $sender;
 	$context{'editor_msg_body'} = $editor_msg->{'msg'}->body_as_string if ($editor_msg) ;
 	
