@@ -1,9 +1,12 @@
 # Conf.pm - This module does the sympa.conf and robot.conf parsing
-# RCS Identication ; $Revision: 7509 $ ; $Date: 2012-07-23 13:00:44 +0200 (lun 23 jui 2012) $ 
+# RCS Identication ; $Revision: 11427 $ ; $Date: 2014-09-24 16:31:44 +0200 (mer. 24 sept. 2014) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
-# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-# Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,8 +19,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 ## This module handles the configuration file for Sympa.
 
@@ -65,11 +67,27 @@ foreach my $hash(@confdef::params){
 
 my %old_params = (
     trusted_ca_options     => 'capath,cafile',
-    msgcat                 => 'localedir',
+    'msgcat'               => '',
     queueexpire            => '',
     clean_delay_queueother => '',
-    dkim_header_list => '',
-    web_recode_to          => 'filesystem_encoding',
+    'dkim_header_list'     => '',
+    web_recode_to          => 'filesystem_encoding', # ??? - 5.2
+    'localedir'            => '',
+    'ldap_export_connection_timeout' => '',          # 3.3b3 - 4.1?
+    'ldap_export_dnmanager' => '',                   # ,,
+    'ldap_export_host'     => '',                    # ,,
+    'ldap_export_name'     => '',                    # ,,
+    'ldap_export_password' => '',                    # ,,
+    'ldap_export_suffix'   => '',                    # ,,
+    'tri'                  => 'sort',                # ??? - 1.3.4-1
+    'sort'                 => '',                    # 1.4.0 - ???
+    'pidfile'              => '',                    # ??? - 6.1.17
+    'pidfile_distribute'   => '',                    # ,,
+    'pidfile_creation'     => '',                    # ,,
+    'pidfile_bulk'         => '',                    # ,,
+    'archived_pidfile'     => '',                    # ,,
+    'bounced_pidfile'      => '',                    # ,,
+    'task_manager_pidfile' => '',                    # ,,
 );
 
 ## These parameters now have a hard-coded value
@@ -267,18 +285,29 @@ sub load {
     }
 		 
     ## Some parameters require CPAN modules
-    if ($Conf{'DKIM_feature'} eq 'on') {
+    if ($Conf{'dkim_feature'} eq 'on') {
         eval "require Mail::DKIM";
         if ($@) {
-            &do_log('err', "Failed to load Mail::DKIM perl module ; setting 'DKIM_feature' to 'off'");
-            $Conf{'DKIM_feature'} = 'off';
+            &do_log('err', "Failed to load Mail::DKIM perl module ; setting 'dkim_feature' to 'off'");
+            $Conf{'dkim_feature'} = 'off';
         }
     }
-    unless ($Conf{'DKIM_feature'} eq 'on'){
-	# dkim_signature_apply_ on nothing if DKIM_feature is off
+    unless ($Conf{'dkim_feature'} eq 'on'){
+	# dkim_signature_apply_ on nothing if dkim_feature is off
 	$Conf{'dkim_signature_apply_on'} = ['']; # empty array
+    }else {
+	$Conf{'dkim_signature_apply_on'} =~ s/\s//g ;
+	my @dkim = split(/,/, $Conf{'dkim_signature_apply_on'});
+	$Conf{'dkim_signature_apply_on'} = \@dkim;
+    }
+    unless ($Conf{'dkim_signer_domain'}) {
+	$Conf{'dkim_signer_domain'} = $Conf{'domain'};
     }
 
+    $Conf{'dmarc_protection_mode'} =~ s/\s//g ;
+    my @dmarc = split(/,/, $Conf{'dmarc_protection_mode'});
+    $Conf{'dmarc_protection_mode'} = \@dmarc;
+    
     ## Load charset.conf file if necessary.
     if($Conf{'legacy_character_support_feature'} eq 'on'){
 	my $charset_conf = &load_charset;
@@ -578,6 +607,12 @@ sub load_robots {
 	## Default for 'host' is the domain
 	$robot_conf->{$robot}{'host'} ||= $robot;
 
+        unless ($robot_conf->{$robot}{'dkim_signer_domain'}) {
+            $robot_conf->{$robot}{'dkim_signer_domain'} = $robot_conf->{$robot}{'domain'};
+        }
+
+	$robot_conf->{$robot}{'dmarc_protection_mode'} ||= $Conf{'dmarc_protection_mode'};
+	
 	$robot_conf->{$robot}{'title'} ||= $wwsconf->{'title'};
 	$robot_conf->{$robot}{'default_home'} ||= $wwsconf->{'default_home'};
 
@@ -1281,7 +1316,7 @@ sub _load_auth {
 sub get_robot_conf {
     my ($robot, $param) = @_;
 
-    if ($robot ne '*') {
+    if (defined $robot && $robot ne '*') {
 	if (defined $Conf{'robots'}{$robot} && defined $Conf{'robots'}{$robot}{$param}) {
 	    return $Conf{'robots'}{$robot}{$param};
 	}
