@@ -1,10 +1,13 @@
 # Lock.pm - This module includes Sympa locking functions
-#<!-- RCS Identication ; $Revision: 7222 $ ; $Date: 2011-10-07 03:06:29 +0200 (ven 07 oct 2011) $ --> 
+#<!-- RCS Identication ; $Revision: 10083 $ ; $Date: 2014-01-01 01:12:12 +0100 (mer. 01 janv. 2014) $ --> 
 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
-# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-# Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,8 +20,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 package Lock;
 
@@ -227,7 +229,7 @@ sub remove_lock {
 	    return undef;
 	}
     }
-    $list_of_locks{$self->{'lock_filename'}}{'fh'} = undef;
+    $list_of_locks{$self->{'lock_filename'}} = undef;
     return 1
 }
 
@@ -330,7 +332,7 @@ sub _lock_nfs {
     &do_log('debug3', "Lock::_lock_nfs($lock_file, $mode, $timeout)");
     
     ## TODO should become a configuration parameter, used with or without NFS
-    my $hold = 30; 
+    my $hold = 60*30; 
     my ($open_mode, $operation);
     
     if ($mode eq 'read') {
@@ -346,8 +348,8 @@ sub _lock_nfs {
     if ($nfs_lock = File::NFSLock->new( {
 	file      => $lock_file,
 	lock_type => $operation|LOCK_NB,
-	blocking_timeout   => $hold,
-	stale_lock_timeout => $timeout,
+	blocking_timeout   => $timeout,
+	stale_lock_timeout => $hold,
     })) {
 	## Read access to prevent "Bad file number" error on Solaris
 	$FH = new FileHandle;
