@@ -1,10 +1,13 @@
 # Message.pm - This module includes Message processing functions
-#<!-- RCS Identication ; $Revision: 7380 $ ; $Date: 2012-05-23 08:02:56 +0200 (mer 23 mai 2012) $ --> 
+#<!-- RCS Identication ; $Revision: 10858 $ ; $Date: 2014-06-06 10:58:13 +0200 (ven. 06 juin 2014) $ --> 
 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
-# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-# Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,10 +20,9 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-=pod 
+=encoding utf-8
 
 =head1 NAME 
 
@@ -150,29 +152,25 @@ sub new {
 	return undef;
     }
     $message->{'msg'} = $msg;
-    $message->{'msg_as_string'} = $msg->as_string;
+    close FILE;
+    
+    unless (open FILE, $file) {
+	&do_log('err', 'Cannot open message file %s : %s',  $file, $!);
+	return undef;
+    }
+    while (<FILE>) {
+	$message->{'msg_as_string'} .= $_;
+    }
+    close FILE;
 
     ## Message size
     $message->{'size'} = -s $file;    
 
     my $hdr = $message->{'msg'}->head;
 
-    ## Extract sender address
-    unless ($hdr->get('From')) {
-	do_log('err', 'No From found in message %s, skipping.', $file);
-	return undef;
-    }   
-    my @sender_hdr = Mail::Address->parse($hdr->get('From'));
-    if ($#sender_hdr == -1) {
-	do_log('err', 'No valid address in From: field in %s, skipping', $file);
-	return undef;
-    }
-    $message->{'sender'} = lc($sender_hdr[0]->address);
-
-    unless (&tools::valid_email($message->{'sender'})) {
-	do_log('err', "Invalid From: field '%s'", $message->{'sender'});
-	return undef;
-    }
+    $message->{'envelope_sender'} = _get_envelope_sender($message);
+    $message->{'sender'} = _get_sender_email($message);
+    return undef unless defined $message->{'sender'};
 
     ## Store decoded subject and its original charset
     my $subject = $hdr->get('Subject');
@@ -307,7 +305,7 @@ sub new {
 		$message->{'smime_subject'} = $signed->{'subject'};
 		do_log('debug', "message %s is signed, signature is checked", $file);
 	    }
-	    ## Il faudrait traiter les cas d'erreur (0 diffrent de undef)
+	    ## Il faudrait traiter les cas d'erreur (0 différent de undef)
 	}
 	
     }
@@ -324,6 +322,84 @@ sub new {
     return $message;
 }
 
+## Get envelope sender (a.k.a. "UNIX From") from Return-Path: header field.
+##
+## We trust in "Return-Path:" header field only at the top of message
+## to prevent forgery.  To ensure it will be added to messages by MDA:
+##
+## - Sendmail:   Add 'P' in the 'F=' flags of local mailer line (such
+##               as 'Mlocal').
+## - Postfix:
+##   - local(8): Available by default.
+##   - pipe(8):  Add 'R' in the 'flags=' attributes in master.cf.
+## - Exim:       Set 'return_path_add' to true with pipe_transport.
+## - qmail:      Use preline(1).
+##
+sub _get_envelope_sender {
+    my $message = shift;
+
+    my $headers = $message->{'msg'}->head->header();
+    my $i = 0;
+    $i++ while $headers->[$i] and $headers->[$i] =~ /^X-Sympa-/;
+    if ($headers->[$i] and $headers->[$i] =~ /^Return-Path:\s*(.+)$/) {
+	my $addr = $1;
+	if ($addr =~ /<>/) { # special: null envelope sender
+	    return '<>';
+	} else {
+	    my @addrs = Mail::Address->parse($addr);
+	    if (@addrs and tools::valid_email($addrs[0]->address)) {
+		return $addrs[0]->address;
+	    }
+	}
+    }
+
+    return undef;
+}
+
+## Get sender of the message according to header fields specified by
+## 'sender_headers' parameter.
+## FIXME: S/MIME signer may not be same as the sender given by this function.
+sub _get_sender_email {
+    my $message = shift;
+
+    my $hdr = $message->{'msg'}->head;
+
+    my $sender = undef;
+    foreach my $field (split /[\s,]+/, $Conf::Conf{'sender_headers'}) {
+	if (lc $field eq 'return-path') {
+	    ## Try to get envelope sender
+	    if ($message->{'envelope_sender'} and
+		$message->{'envelope_sender'} ne '<>') {
+		$sender = lc($message->{'envelope_sender'});
+	    }
+	} elsif ($hdr->get($field)) {
+	    ## Try to get message header.
+	    ## On "Resent-*:" headers, the first occurrence must be used (see
+	    ## RFC 5322 3.6.6).
+	    ## FIXME: Though "From:" can occur multiple times, only the first
+	    ## one is detected.
+	    my $addr = $hdr->get($field, 0); # get the first one
+	    my @sender_hdr = Mail::Address->parse($addr);
+	    if (@sender_hdr and $sender_hdr[0]->address) {
+		$sender = lc($sender_hdr[0]->address);
+		last;
+	    }
+	}
+
+	last if defined $sender;
+    }
+    unless (defined $sender) {
+	do_log('err', 'No valid sender address');
+	return undef;
+    }
+    unless (tools::valid_email($sender)) {
+	do_log('err', 'Invalid sender address "%s"', $sender);
+	return undef;
+    }
+
+    return $sender;
+}
+
 =pod 
 
 =head2 sub dump
@@ -474,27 +550,25 @@ sub get_topic {
 
 sub clean_html {
     my $self = shift;
-    my ($listname, $robot) = split(/\@/,$self->{'rcpt'});
-    $robot = lc($robot);
-    $listname = lc($listname);
-    $robot ||= $Conf::Conf{'host'};
+    my $robot = shift;
     my $new_msg;
-    if($new_msg = &fix_html_part($self->{'msg'},$robot)) {
+    if($new_msg = _fix_html_part($self->{'msg'}, $robot)) {
 	$self->{'msg'} = $new_msg;
 	return 1;
     }
     return 0;
 }
 
-sub fix_html_part {
+sub _fix_html_part {
     my $part = shift;
     my $robot = shift;
     return $part unless $part;
+
     my $eff_type = $part->head->mime_attr("Content-Type");
     if ($part->parts) {
 	my @newparts = ();
 	foreach ($part->parts) {
-	    push @newparts, &fix_html_part($_,$robot);
+	    push @newparts, _fix_html_part($_, $robot);
 	}
 	$part->parts(\@newparts);
     } elsif ($eff_type =~ /^text\/html/i) {
@@ -503,22 +577,25 @@ sub fix_html_part {
 	return $part if !$bodyh or $bodyh->is_encoded;
 
 	my $body = $bodyh->as_string;
-	# Re-encode parts with 7-bit charset (ISO-2022-*), since
-	# StripScripts cannot handle them correctly.
+	# Re-encode parts to UTF-8, since StripScripts cannot handle texts
+	# with some charsets (ISO-2022-*, UTF-16*, ...) correctly.
 	my $cset = MIME::Charset->new($part->head->mime_attr('Content-Type.Charset') || '');
 	unless ($cset->decoder) {
 	    # Charset is unknown.  Detect 7-bit charset.
 	    my ($dummy, $charset) =
 		MIME::Charset::body_encode($body, '', Detect7Bit => 'YES');
-	    $cset = MIME::Charset->new($charset);
+	    $cset = MIME::Charset->new($charset)
+		if $charset;
 	}
-	if ($cset->decoder and $cset->as_string =~ /^ISO-2022-/i) {
-	    $part->head->mime_attr('Content-Type.Charset', 'UTF-8');
+	if ($cset->decoder and
+	    $cset->as_string ne 'UTF-8' and $cset->as_string ne 'US-ASCII') {
 	    $cset->encoder('UTF-8');
 	    $body = $cset->encode($body);
+	    $part->head->mime_attr('Content-Type.Charset', 'UTF-8');
 	}
 
-	my $filtered_body = &tools::sanitize_html('string' => $body, 'robot'=> $robot);
+	my $filtered_body = tools::sanitize_html(
+	    'string' => $body, 'robot'=> $robot);
 
 	my $io = $bodyh->open("w");
 	unless (defined $io) {
@@ -527,6 +604,7 @@ sub fix_html_part {
 	}
 	$io->print($filtered_body);
 	$io->close;
+	$part->sync_headers(Length => 'COMPUTE');
     }
     return $part;
 }
@@ -542,7 +620,7 @@ sub fix_html_part {
 
 =item * Serge Aumont <sa AT cru.fr> 
 
-=item * Olivier Salan <os AT cru.fr> 
+=item * Olivier SalaE<252>n <os AT cru.fr> 
 
 =back 
 
