@@ -1,10 +1,13 @@
 # SQLSource.pm - This module includes SQL DB related functions
-#<!-- RCS Identication ; $Revision: 7598 $ --> 
+#<!-- RCS Identication ; $Revision: 10083 $ --> 
 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
-# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-# Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,14 +20,14 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Softwarec
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 package SQLSource;
 
 use strict;
 
 use Carp;
+use DBI;
 use Exporter;
 
 use Conf;
@@ -48,7 +51,7 @@ our %date_format = (
 		   'write' => {
 		       'Pg' => '\'epoch\'::timestamp with time zone + \'%d sec\'',
 		       'mysql' => 'FROM_UNIXTIME(%d)',
-		       'Oracle' => 'to_date(to_char(round(%s/86400) + to_number(to_char(to_date(\'01/01/1970\',\'dd/mm/yyyy\'), \'J\'))) || \':\' ||to_char(mod(%s,86400)), \'J:SSSSS\')',
+		       'Oracle' => 'to_date(to_char(floor(%s/86400) + to_number(to_char(to_date(\'01/01/1970\',\'dd/mm/yyyy\'), \'J\'))) || \':\' ||to_char(mod(%s,86400)), \'J:SSSSS\')',
 		       'Sybase' => 'dateadd(second,%s,\'01/01/1970\')',
 		       'SQLite' => 'datetime(%d,\'unixepoch\',\'localtime\')'
 		       }
@@ -86,12 +89,6 @@ sub connect {
     	$param = $self->{'param'};
     }
     
-    unless ( eval "require DBI" ){
-	do_log('err',"Unable to use DBI library, install DBI (CPAN) first");
-	return undef ;
-    }
-    require DBI;
-
     $param->{'db_host'} ||= $param->{'host'};
     $param->{'db_user'} ||= $param->{'user'};
     $param->{'db_passwd'} ||= $param->{'passwd'};
@@ -104,9 +101,18 @@ sub connect {
 	    return undef;
 	}
     }
-    ## SQLite just need a db_name
-    unless ($param->{'db_type'} eq 'SQLite') {
-	foreach my $db_param ('db_type','db_name','db_host','db_user') {
+    if ($param->{'db_type'} eq 'SQLite') {
+        ## SQLite just need a db_name
+    } elsif ($param->{'db_type'} eq 'ODBC') {
+        ## ODBC just needs a db_name, db_user
+	foreach my $db_param ('db_user','db_passwd') {
+	    unless ($param->{$db_param}) {
+		do_log('info','Missing parameter %s for DBI connection', $db_param);
+		return undef;
+	    }
+	}
+    } else {
+	foreach my $db_param ('db_host','db_user','db_passwd') {
 	    unless ($param->{$db_param}) {
 		do_log('info','Missing parameter %s for DBI connection', $db_param);
 		return undef;
@@ -138,6 +144,9 @@ sub connect {
 	$connect_string = "DBI:SQLite:dbname=$param->{'db_name'}";
     }elsif ($param->{'db_type'} eq 'Informix') {
         $connect_string = "DBI:Informix:".$param->{'db_name'}."@".$param->{'db_host'}; 
+    # ODBC support
+    }elsif ($param->{'db_type'} eq 'ODBC') {
+	$connect_string = "DBI:ODBC:".$param->{'db_name'};
     }else {
 	$connect_string = "DBI:$param->{'db_type'}:$param->{'db_name'}:$param->{'db_host'}";
     }
@@ -150,7 +159,10 @@ sub connect {
     }
  
     ## First check if we have an active connection with this server
+    ## We require that user also matches (except SQLite).
     if (defined $db_connections{$connect_string} && 
+	($param->{'db_type'} eq 'SQLite' or
+	 $db_connections{$connect_string}{'db_user'} eq $param->{'db_user'}) &&
 	defined $db_connections{$connect_string}{'dbh'} && 
 	$db_connections{$connect_string}{'dbh'}->ping()) {
       
@@ -162,6 +174,21 @@ sub connect {
       
       ## Set environment variables
       ## Used by Oracle (ORACLE_HOME)
+
+      ## Client encoding derived from the environment variable.
+      ## Set this before parsing db_env to allow override if one knows what
+      ## she is doing.
+      ## Note: on mysql and Pg, "SET NAMES" will be executed below; on SQLite,
+      ## no need to set encoding.
+      if ($param->{'db_type'} eq 'Oracle') {
+	## NLS_LANG.  This needs to be set before connecting, otherwise it's
+	## useless.  Underscore (_) and dot (.) are a vital part as NLS_LANG
+	## has the syntax "language_territory.charset".
+	$ENV{'NLS_LANG'} = '_.UTF8';
+      } elsif ($param->{'db_type'} eq 'Sybase') {
+	$ENV{'SYBASE_CHARSET'} = 'utf8';
+      }
+
       if ($param->{'db_env'}) {
 	foreach my $env (split /;/,$param->{'db_env'}) {
 	  my ($key, $value) = split /=/, $env;
@@ -211,19 +238,29 @@ sub connect {
 	}
 	
       }
-      
-      if ($param->{'db_type'} eq 'Pg') { # Configure Postgres to use ISO format dates
-	$dbh->do ("SET DATESTYLE TO 'ISO';");
+
+      # mysql:
+      # - At first, reset "mysql_auto_reconnect" driver attribute.
+      #   connect() sets it to true not according to \%attr argument
+      #   when the processes are running under mod_perl or CGI environment so
+      #   that "SET NAMES utf8" will be skipped.
+      # - Set client-side character set to "utf8" or "utf8mb4".
+      if ($param->{'db_type'} eq 'mysql') {
+	    $dbh->{'mysql_auto_reconnect'} = 0;
+
+	    unless (defined $dbh->do("SET NAMES 'utf8mb4'") or
+		defined $dbh->do("SET NAMES 'utf8'")) {
+		do_log('err', 'Cannot set client-side character set: %s',
+		    $dbh->errstr);
+	    }
       }
-      
-      ## Set client encoding to UTF8
-      if ($param->{'db_type'} eq 'mysql' ||
-	  $param->{'db_type'} eq 'Pg') {
+
+      # Pg:
+      # - Configure Postgres to use ISO format dates.
+      # - Set client encoding to UTF8.
+      if ($param->{'db_type'} eq 'Pg') {
+	$dbh->do ("SET DATESTYLE TO 'ISO';");
 	$dbh->do("SET NAMES 'utf8'");
-      }elsif ($param->{'db_type'} eq 'oracle') { 
-	$ENV{'NLS_LANG'} = 'UTF8';
-      }elsif ($param->{'db_type'} eq 'Sybase') { 
-	$ENV{'SYBASE_CHARSET'} = 'utf8';
       }
       
       ## added sybase support
@@ -245,6 +282,7 @@ sub connect {
       $self->{'dbh'} = $dbh if $self;
       $self->{'connect_string'} = $connect_string if $self;     
       $db_connections{$connect_string}{'dbh'} = $dbh;
+      $db_connections{$connect_string}{'db_user'} = $param->{'db_user'};
       
       do_log('debug2','Connected to Database %s',$param->{'db_name'});
       return $dbh;
