@@ -1,9 +1,12 @@
 # admin.pm - This module includes administrative function for the lists
-# RCS Identication ; $Revision: 7327 $ ; $Date: 2012-04-17 05:59:10 +0200 (mar 17 avr 2012) $ 
+# RCS Identication ; $Revision: 10193 $ ; $Date: 2014-01-23 15:07:50 +0100 (jeu. 23 janv. 2014) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
-# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-# Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,8 +19,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 =pod 
 
@@ -41,6 +43,7 @@ use Language;
 use Log;
 use tools;
 use Sympa::Constants;
+use Data::Dumper;
 
 =pod 
 
@@ -148,6 +151,7 @@ Creates a list. Used by the create_list() sub in sympa.pl and the do_create_list
 #       - $template : the create list template 
 #       - $robot : the list's robot       
 #       - $origin : the source of the command : web, soap or command_line  
+#              no longer used
 # OUT : - hash with keys :
 #          -list :$list
 #          -aliases : undef if not applicable; 1 (if ok) or
@@ -200,19 +204,23 @@ sub create_list_old{
 	}
     }    
 
+    if ($param->{'listname'} eq &Conf::get_robot_conf($robot,'email')) {
+	&do_log('err','admin::create_list : incorrect listname %s matches one of service aliases', $param->{'listname'});
+	return undef;
+    }
+
     ## Check listname on SMTP server
-    my $res = &list_check_smtp($param->{'listname'});
+    my $res = &list_check_smtp($param->{'listname'}, $robot);
     unless (defined $res) {
-	&do_log('err', "admin::create_list_old : can't check list %.128s on %.128s",
-		$param->{'listname'},
-		$Conf::Conf{'list_check_smtp'});
+	&do_log('err', "admin::create_list_old : can't check list %.128s on %s",
+		$param->{'listname'}, $robot);
 	return undef;
     }
     
     ## Check this listname doesn't exist already.
     if( $res || new List ($param->{'listname'}, $robot, {'just_try' => 1})) {
-	&do_log('err', 'admin::create_list_old : could not create already existing list %s for ', 
-		$param->{'listname'});
+	&do_log('err', 'admin::create_list_old : could not create already existing list %s on %s for ', 
+		$param->{'listname'}, $robot);
 	foreach my $o (@{$param->{'owner'}}){
 	    &do_log('err',$o->{'email'});
 	}
@@ -275,11 +283,10 @@ sub create_list_old{
     unless ($lock->lock('write')) {
 	return undef;
     }
-    if($origin eq "command_line") {
-	open CONFIG, '>:utf8', "$list_dir/config";
-    }
-    else {
-	open CONFIG, '>:bytes', "$list_dir/config";
+    unless (open CONFIG, '>', "$list_dir/config") {
+	&do_log('err','Impossible to create %s/config : %s', $list_dir, $!);
+	$lock->unlock();
+	return undef;
     }
     ## Use an intermediate handler to encode to filesystem_encoding
     my $config = '';
@@ -295,18 +302,11 @@ sub create_list_old{
 
     ## Creation of the info file 
     # remove DOS linefeeds (^M) that cause problems with Outlook 98, AOL, and EIMS:
-    $param->{'description'} =~ s/\015//g;
+    $param->{'description'} =~ s/\r\n|\r/\n/g;
 
-    ## info file creation. Use UTF-8 for command line orgigin only.
-    if($origin eq "command_line") {
-	unless (open INFO, '>:utf8', "$list_dir/info") {
-	    &do_log('err','Impossible to create %s/info : %s',$list_dir,$!);
-	}
-    }
-    else {
-	unless (open INFO, '>:bytes', "$list_dir/info") {
-	    &do_log('err','Impossible to create %s/info : %s',$list_dir,$!);
-	}
+    ## info file creation.
+    unless (open INFO, '>', "$list_dir/info") {
+	&do_log('err','Impossible to create %s/info : %s', $list_dir, $!);
     }
     if (defined $param->{'description'}) {
 	Encode::from_to($param->{'description'}, 'utf8', $Conf::Conf{'filesystem_encoding'});
@@ -329,12 +329,11 @@ sub create_list_old{
     my $return = {};
     $return->{'list'} = $list;
 
-    if ($param->{'status'} eq 'open') {
+    if ($list->{'admin'}{'status'} eq 'open') {
 	$return->{'aliases'} = &install_aliases($list,$robot);
-	return $return;
-    }
-
+    }else{
     $return->{'aliases'} = 1;
+    }
 
     ## Synchronize list members if required
     if ($list->has_include_data_sources()) {
@@ -342,7 +341,7 @@ sub create_list_old{
 	$list->sync_include();
     }
     
-    $list->save_config;
+    $list->save_config($param->{'creation_email'});
 
     return $return;
 }
@@ -411,18 +410,21 @@ sub create_list{
 	    return undef;
 	}
     }    
+    if ($param->{'listname'} eq &Conf::get_robot_conf($robot,'email')) {
+	&do_log('err','admin::create_list : incorrect listname %s matches one of service aliases', $param->{'listname'});
+	return undef;
+    }
 
     ## Check listname on SMTP server
-    my $res = &list_check_smtp($param->{'listname'});
+    my $res = &list_check_smtp($param->{'listname'}, $robot);
     unless (defined $res) {
-	&do_log('err', "admin::create_list : can't check list %.128s on %.128s",
-		$param->{'listname'},
-		$Conf::Conf{'list_check_smtp'});
+	&do_log('err', "admin::create_list : can't check list %.128s on %s",
+		$param->{'listname'}, $robot);
 	return undef;
     }
 
     if ($res) {
-	&do_log('err', 'admin::create_list : could not create already existing list %s for ',$param->{'listname'});
+	&do_log('err', 'admin::create_list : could not create already existing list %s on %s for ', $param->{'listname'}, $robot);
 	foreach my $o (@{$param->{'owner'}}){
 	    &do_log('err',$o->{'email'});
 	}
@@ -436,6 +438,8 @@ sub create_list{
 	return undef;
     }
 
+    my $family_config = &Conf::get_robot_conf($robot,'automatic_list_families');
+    $param->{'family_config'} = $family_config->{$family->{'name'}};
     my $conf;
     my $tt_result = &tt2::parse_tt2($param, 'config.tt2', \$conf, [$family->{'dir'}]);
     unless (defined $tt_result || !$abort_on_error) {
@@ -483,7 +487,11 @@ sub create_list{
     }
 
     ## Creation of the config file
-    open CONFIG, '>:utf8', "$list_dir/config";
+    unless (open CONFIG, '>', "$list_dir/config") {
+	&do_log('err','Impossible to create %s/config : %s', $list_dir, $!);
+	$lock->unlock();
+	return undef;
+    }
     #&tt2::parse_tt2($param, 'config.tt2', \*CONFIG, [$family->{'dir'}]);
     print CONFIG $conf;
     close CONFIG;
@@ -493,10 +501,10 @@ sub create_list{
 
     ## Creation of the info file 
     # remove DOS linefeeds (^M) that cause problems with Outlook 98, AOL, and EIMS:
-    $param->{'description'} =~ s/\015//g;
+    $param->{'description'} =~ s/\r\n|\r/\n/g;
     
-    unless (open INFO, '>:utf8', "$list_dir/info") {
-	&do_log('err','Impossible to create %s/info : %s',$list_dir,$!);
+    unless (open INFO, '>', "$list_dir/info") {
+	&do_log('err','Impossible to create %s/info : %s', $list_dir, $!);
     }
     if (defined $param->{'description'}) {
 	print INFO $param->{'description'};
@@ -504,7 +512,12 @@ sub create_list{
     close INFO;
 
     ## Create associated files if a template was given.
-    for my $file ('message.footer','message.header','message.footer.mime','message.header.mime','info') {
+    my @files_to_parse;
+    foreach my $file (split ',',Conf::get_robot_conf($robot,'parsed_family_files')) {
+	$file =~ s{\s}{}g;
+	push @files_to_parse,$file;
+    }
+    for my $file (@files_to_parse) {
 	my $template_file = &tools::get_filename('etc',{},$file.".tt2", $robot,$family);
 	if (defined $template_file) {
 	    my $file_content;
@@ -513,7 +526,7 @@ sub create_list{
 		&do_log('err', 'admin::create_list : tt2 error. List %s from family %s@%s, file %s',
 			$param->{'listname'}, $family->{'name'},$robot,$file);
 	    }
-	    unless (open FILE, '>:utf8', "$list_dir/$file") {
+	    unless (open FILE, '>', "$list_dir/$file") {
 		&do_log('err','Impossible to create %s/%s : %s',$list_dir,$file,$!);
 	    }
 	    print FILE $file_content;
@@ -553,10 +566,9 @@ sub create_list{
 
     if ($list->{'admin'}{'status'} eq 'open') {
 	$return->{'aliases'} = &install_aliases($list,$robot);
-	return $return;
-    }
-
+    }else{
     $return->{'aliases'} = 1;
+    }
 
     ## Synchronize list members if required
     if ($list->has_include_data_sources()) {
@@ -625,7 +637,11 @@ sub update_list{
     }
 
     ## Creation of the config file
-    open CONFIG, '>:utf8', "$list->{'dir'}/config";
+    unless (open CONFIG, '>', "$list->{'dir'}/config") {
+	&do_log('err','Impossible to create %s/config : %s', $list->{'dir'}, $!);
+	$lock->unlock();
+	return undef;
+    }
     &tt2::parse_tt2($param, 'config.tt2', \*CONFIG, [$family->{'dir'}]);
     close CONFIG;
 
@@ -654,6 +670,29 @@ sub update_list{
     }
     $list->{'admin'}{'family_name'} = $family->{'name'};
 
+   ## Create associated files if a template was given.
+    my @files_to_parse;
+    foreach my $file (split ',',Conf::get_robot_conf($robot,'parsed_family_files')) {
+	$file =~ s{\s}{}g;
+	push @files_to_parse,$file;
+    }
+    for my $file (@files_to_parse) {
+	my $template_file = &tools::get_filename('etc',{},$file.".tt2", $robot,$family);
+	if (defined $template_file) {
+	    my $file_content;
+	    my $tt_result = &tt2::parse_tt2($param, $file.".tt2", \$file_content, [$family->{'dir'}]);
+	    unless (defined $tt_result) {
+		&do_log('err', 'admin::create_list : tt2 error. List %s from family %s@%s, file %s',
+			$param->{'listname'}, $family->{'name'},$robot,$file);
+	    }
+	    unless (open FILE, '>', "$list->{'dir'}/$file") {
+		&do_log('err','Impossible to create %s/%s : %s',$list->{'dir'},$file,$!);
+	    }
+	    print FILE $file_content;
+	    close FILE;
+	}
+    }
+
     ## Synchronize list members if required
     if ($list->has_include_data_sources()) {
 	&do_log('notice', "Synchronizing list members...");
@@ -728,14 +767,15 @@ sub rename_list{
     ## Check listname on SMTP server
     my $res = list_check_smtp($param{'new_listname'}, $param{'new_robot'});
     unless ( defined($res) ) {
-      &do_log('err', "can't check list %.128s on %.128s", $param{'new_listname'}, &Conf::get_robot_conf($param{'new_robot'}, 'list_check_smtp'));
+      &do_log('err', "can't check list %.128s on %.128s",
+	      $param{'new_listname'}, $param{'new_robot'});
       return 'internal';
     }
 
     if( $res || 
 	($list->{'name'} ne $param{'new_listname'}) && ## Do not test if listname did not change
 	(new List ($param{'new_listname'}, $param{'new_robot'}, {'just_try' => 1}))) {
-      &do_log('err', 'Could not rename list %s : new list %s already existing list', $list->{'name'}, $param{'new_listname'});
+      &do_log('err', 'Could not rename list %s on %s: new list %s on %s already existing list', $list->{'name'}, $robot, $param{'new_listname'}, $param{'new_robot'});
       return 'list_already_exists';
     }
     
@@ -1104,12 +1144,14 @@ sub check_owner_defined {
      my $smtp;
      my (@suf, @addresses);
 
-     my $smtp_relay = $Conf::Conf{'robots'}{$robot}{'list_check_smtp'} || $Conf::Conf{'list_check_smtp'};
-     my $suffixes = $Conf::Conf{'robots'}{$robot}{'list_check_suffixes'} || $Conf::Conf{'list_check_suffixes'};
+     my $smtp_relay = &Conf::get_robot_conf($robot, 'list_check_smtp');
+     my $smtp_helo = &Conf::get_robot_conf($robot, 'list_check_helo') || $smtp_relay;
+     $smtp_helo =~ s/:[-\w]+$//;
+     my $suffixes = &Conf::get_robot_conf($robot, 'list_check_suffixes');
      return 0 
 	 unless ($smtp_relay && $suffixes);
      my $domain = &Conf::get_robot_conf($robot, 'host');
-     &do_log('debug2', 'list_check_smtp(%s)',$list);
+     &do_log('debug2', 'list_check_smtp(%s,%s)', $list, $robot);
      @suf = split(/,/,$suffixes);
      return 0 if ! @suf;
      for(@suf) {
@@ -1122,7 +1164,7 @@ sub check_owner_defined {
 	 return undef;
      }
      if( $smtp = Net::SMTP->new($smtp_relay,
-				Hello => $smtp_relay,
+				Hello => $smtp_helo,
 				Timeout => 30) ) {
 	 $smtp->mail('');
 	 for(@addresses) {
@@ -1152,7 +1194,7 @@ sub install_aliases {
     &do_log('debug', "admin::install_aliases($list->{'name'},$robot)");
 
     return 1
-	if ($Conf::Conf{'sendmail_aliases'} =~ /^none$/i);
+	if Conf::get_robot_conf($list->{'domain'}, 'sendmail_aliases') =~ /^none$/i;
 
     my $alias_installed = 0;
     my $alias_manager = $Conf::Conf{'alias_manager' };
@@ -1216,7 +1258,7 @@ sub install_aliases {
      &do_log('info', "_remove_aliases($list->{'name'},$robot");
 
     return 1
-	if ($Conf::Conf{'sendmail_aliases'} =~ /^none$/i);
+	if Conf::get_robot_conf($list->{'domain'}, 'sendmail_aliases') =~ /^none$/i;
 
      my $status = $list->remove_aliases();
      my $suffix = &Conf::get_robot_conf($robot, 'return_path_suffix');
