@@ -1,9 +1,12 @@
 # mail.pm - This module includes mail sending functions and does the smtp job.
-# RCS Identication ; $Revision: 7551 $ ; $Date: 2012-08-04 12:02:48 +0200 (sam 04 aoû 2012) $ 
+# RCS Identication ; $Revision: 10713 $ ; $Date: 2014-05-23 11:18:40 +0200 (ven. 23 mai 2014) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
-# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-# Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,8 +19,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 package mail;
 
@@ -43,7 +45,7 @@ our @EXPORT = qw(mail_file mail_message mail_forward set_send_spool);
 
 
 ## RCS identification.
-#my $id = '@(#)$Id: mail.pm 7551 2012-08-04 10:02:48Z sikeda $';
+#my $id = '@(#)$Id: mail.pm 10713 2014-05-23 09:18:40Z sikeda $';
 
 my $opensmtp = 0;
 my $fh = 'fh0000000000';	## File handle for the stream.
@@ -153,9 +155,15 @@ sub mail_file {
 
     }else { # or not
 	$message .= $data->{'body'};
-       }
-       
-    ## ## Does the message include headers ?
+    }
+
+    # Does the message include headers ?
+    if ($data->{'headers'}) {
+        foreach my $field (keys %{$data->{'headers'}}) {
+            $field =~ tr/A-Z/a-z/;
+            $header_ok{$field} = 1;
+        }
+    }
     if ($header_possible) {
 	
 	foreach my $line (split(/\n/,$message)) {
@@ -169,8 +177,11 @@ sub mail_file {
 		last;
 	    }
 		
-	    foreach my $header ('date', 'to','from','subject','reply-to','mime-version', 'content-type','content-transfer-encoding') {
-		if ($line=~/^$header:/i) {
+	    foreach my $header (
+		qw(message-id date to from subject reply-to
+		mime-version content-type content-transfer-encoding)
+	    ) {
+		if ($line=~/^$header\s*:/i) {
 		    $header_ok{$header} = 1;
 		    last;
 		}
@@ -181,6 +192,10 @@ sub mail_file {
     ## ADD MISSING HEADERS
     my $headers="";
 
+    unless ($header_ok{'message-id'}) {
+	$headers .= sprintf("Message-Id: %s\n", tools::get_message_id($robot));
+    }
+
     unless ($header_ok{'date'}) {
 	my $now = time;
 	my $tzoff = timegm(localtime $now) - $now;
@@ -288,8 +303,10 @@ sub mail_file {
     }
 
     my $listname = ''; 
-    if (ref($data->{'list'}) eq "HASH") {
+    if (ref($data->{'list'}) eq "HASH") { # compatibility
 	$listname = $data->{'list'}{'name'};
+    } elsif (ref($data->{'list'}) eq 'List') {
+	$listname = $data->{'list'}->{'name'};
     } elsif ($data->{'list'}) {
 	$listname = $data->{'list'};
     }
@@ -342,6 +359,8 @@ sub mail_message {
     my $dkim  =  $params{'dkim_parameters'};
     my $tag_as_last = $params{'tag_as_last'};
 
+    my $merge = $list->{'admin'}{'merge_feature'};
+
     my $host = $list->{'admin'}{'host'};
     my $robot = $list->{'domain'};
 
@@ -381,6 +400,8 @@ sub mail_message {
 	close (MSG);
     }
     
+    $merge = 0 if ($message->{'protected'});
+    
     my %rcpt_by_dom ;
 
     my @sendto;
@@ -449,7 +470,7 @@ sub mail_message {
 				'use_bulk' => 1,
 				'verp' => $verp,
 				'dkim' => $dkim,
-				'merge' => $list->{'admin'}{'merge_feature'},
+				'merge' => $merge,
 				'tag_as_last' => $tag_as_last));
 }
 
@@ -592,7 +613,7 @@ sub sendto {
 	    }
 	    my $encrypted_msg_as_string;
 	    if ($encrypted_msg_as_string = &tools::smime_encrypt ($msg_header, $msg_body, $email)){
-		&sending('msg' => $encrypted_msg_as_string,
+		my $result = &sending('msg' => $encrypted_msg_as_string,
 			 'rcpt' => $email,
 			 'from' => $from,
 			 'listname' => $listname,
@@ -601,6 +622,10 @@ sub sendto {
 			 'delivery_date' =>  $delivery_date,
 			 'use_bulk' => $use_bulk,
 			 'tag_as_last' => $tag_as_last);
+		return $result;
+	    }else{
+		Log::do_log('err','Unable to encrypt message to list %s for recipient %s',$listname,$email);
+		return undef;
 	    }
 	    $tag_as_last = 0;
 	}
@@ -622,6 +647,7 @@ sub sendto {
 				  'tag_as_last' => $tag_as_last);
 	    return $result;
 	}else{
+	    Log::do_log('err','Unable to get string from message to list %s',$listname);
 	    return undef;
 	}   
     }
@@ -661,7 +687,7 @@ sub sending {
     my $sign_mode = $params{'sign_mode'};
     my $sympa_email =  $params{'sympa_email'};
     my $priority_message =  $params{'priority'};
-    my $priority_packet = $Conf::Conf{'sympa_packet_priority'};
+    my $priority_packet = Conf::get_robot_conf($robot, 'sympa_packet_priority');
     my $delivery_date = $params{'delivery_date'};
     $delivery_date = time() unless ($delivery_date); 
     my $verp  =  $params{'verp'};
@@ -1013,13 +1039,15 @@ sub fix_part($$$$) {
     my $defcharset = shift;
     return $part unless $part;
 
-    my $enc = $part->head->mime_attr("Content-Transfer-Encoding");
+    my $enc = $part->head->mime_encoding;
     # Parts with nonstandard encodings aren't modified.
     return $part
 	if $enc and $enc !~ /^(?:base64|quoted-printable|[78]bit|binary)$/i;
-
     my $eff_type = $part->effective_type;
-    return $part if $eff_type =~ m{^multipart/(signed|encrypted)$};
+    # Signed or encrypted parts aren't modified.
+    if ($eff_type =~ m{^multipart/(signed|encrypted)$}){
+	return $part;
+    }
 
     if ($part->head->get('X-Sympa-Attach')) { # Need re-attaching data.
 	my $data = shift @{$attachments};
@@ -1069,6 +1097,10 @@ sub fix_part($$$$) {
 	    return $part;
 	}
 
+	## normalize newline to CRLF if transfer-encoding is BASE64.
+	$newbody =~ s/\r\n|\r|\n/\r\n/g
+	    if $newenc and $newenc eq 'BASE64';
+
 	# Fix headers and body.
 	$head->mime_attr("Content-Type", "TEXT/PLAIN")
 	    unless $head->mime_attr("Content-Type");
