@@ -1,8 +1,11 @@
-# $Id: tt2.pm 6365 2010-03-12 13:49:48Z david.verdin $
+# $Id: tt2.pm 10083 2014-01-01 00:12:12Z sikeda $
 #
 # Sympa - SYsteme de Multi-Postage Automatique
-# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-# Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,8 +18,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # TT2 adapter for sympa's template system - Chia-liang Kao <clkao@clkao.org>
 # usage: replace require 'parser.pl' in wwwsympa and other .pl
@@ -177,6 +179,26 @@ sub wrap {
     };
 }
 
+# IN:
+#    $context: Context.
+#    $type: type of list parameter value: 'reception', 'visibility', 'status'
+#        or others (default).
+#    $withval: if parameter value is added to the description. False by
+#        default.
+# OUT:
+#    Subref to generate i18n'ed description of list parameter value.
+sub optdesc {
+    my ($context, $type, $withval) = @_;
+    return sub {
+	my $x = shift;
+	return undef unless defined $x;
+	return undef unless $x =~ /\S/;
+	$x =~ s/^\s+//;
+	$x =~ s/\s+$//;
+	return List->get_option_title($x, $type, $withval);
+    };
+}
+
 ## To add a directory to the TT2 include_path
 sub add_include_path {
     my $path = shift;
@@ -189,6 +211,11 @@ sub get_include_path {
     return @other_include_path;
 }
 
+## Clear current INCLUDE_PATH
+sub clear_include_path {
+    @other_include_path = ();
+}
+
 ## Allow inclusion/insertion of file with absolute path
 sub allow_absolute_path {
     $allow_absolute = 1;
@@ -209,10 +236,11 @@ sub get_error {
 sub parse_tt2 {
     my ($data, $template, $output, $include_path, $options) = @_;
     $include_path ||= [Sympa::Constants::DEFAULTDIR];
+    $options ||= {};
 
     ## Add directories that may have been added
     push @{$include_path}, @other_include_path;
-    @other_include_path = (); ## Reset it
+    clear_include_path(); ## Reset it
 
     my $wantarray;
 
@@ -236,6 +264,7 @@ sub parse_tt2 {
 	    helploc => [\&tt2::maketext, 1],
 	    locdt => [\&tt2::locdatetime, 1],
 	    wrap => [\&tt2::wrap, 1],
+	    optdesc => [\&tt2::optdesc, 1],
 	    qencode => [\&qencode, 0],
  	    escape_xml => [\&escape_xml, 0],
 	    escape_url => [\&escape_url, 0],
@@ -252,13 +281,20 @@ sub parse_tt2 {
 	$config->{'ABSOLUTE'} = 1;
 	$allow_absolute = 0;
     }
+    if ($options->{'has_header'}) { # body is separated by an empty line.
+	if (ref $template) {
+	    $template = \("\n" . $$template);
+	} else {
+	    $template = \"\n[% PROCESS $template %]";
+	}
+    }
 
     my $tt2 = Template->new($config) or die "Template error: ".Template->error();
 
     unless ($tt2->process($template, $data, $output)) {
 	$last_error = $tt2->error();
-	&do_log('err', 'Failed to parse %s : %s', $template, $tt2->error());
-	&do_log('err', 'Looking for TT2 files in %s', join(',',@{$include_path}));
+	&Log::do_log('err', 'Failed to parse %s : %s', $template, $last_error->as_string);
+	&Log::do_log('err', 'Looking for TT2 files in %s', join(',',@{$include_path}));
 
 
 	return undef;
