@@ -1,23 +1,26 @@
 /* queue.c - This program does the messages spooling
-   RCS Identication ; $Revision: 6491 $ ; $Date: 2010-06-09 11:50:02 +0200 (mer 09 jun 2010) $ 
+  RCS Identication ; $Revision: 10083 $ ; $Date: 2014-01-01 01:12:12 +0100 (mer. 01 janv. 2014) $ 
 
-   Sympa - SYsteme de Multi-Postage Automatique
-   Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-   Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+  Sympa - SYsteme de Multi-Postage Automatique
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+  Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+  Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+  2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+  Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -25,8 +28,12 @@
 #include <sysexits.h>
 #include <string.h>
 #include <stdlib.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <time.h>
 
-static char rcsid[] = "(@)$Id: queue.c 6491 2010-06-09 09:50:02Z sympa-authors $";
+static char rcsid[] = "(@)$Id: queue.c 10083 2014-01-01 00:12:12Z sikeda $";
 
 static char     qfile[128];
 static char     buf[16384];
