@@ -1,8 +1,11 @@
 # SympaSession.pm - This module includes functions managing HTTP sessions in Sympa
 #
 # Sympa - SYsteme de Multi-Postage Automatique
-# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
-# Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+#
+# Copyright (c) 1997, 1998, 1999 Institut Pasteur & Christophe Wolfhugel
+# Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+# 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
+# Copyright (c) 2011, 2012, 2013, 2014 GIP RENATER
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,8 +18,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
 package SympaSession;
@@ -33,7 +35,9 @@ use Conf;
 
 # this structure is used to define which session attributes are stored in a dedicated database col where others are compiled in col 'data_session'
 my %session_hard_attributes = ('id_session' => 1, 
+			       'prev_id' => 1,
 			       'date' => 1, 
+			       'refresh_date' => 1,
 			       'remote_addr'  => 1,
 			       'robot'  => 1,
 			       'email' => 1, 
@@ -54,8 +58,8 @@ sub new {
 
     do_log('debug', 'SympaSession::new(%s,%s,%s)', $robot,$cookie,$action);
 
-    my $session={};
-    bless $session, $pkg;
+    my $self = {'robot' => $robot}; # set current robot
+    bless $self, $pkg;
     
     unless ($robot) {
 	&do_log('err', 'Missing robot parameter, cannot create session object') ;
@@ -65,15 +69,15 @@ sub new {
 #   passive_session are session not stored in the database, they are used for crawler bots and action such as css, wsdl and rss
     
     if (&tools::is_a_crawler($robot,{'user_agent_string' => $ENV{'HTTP_USER_AGENT'}})) {
-	$session->{'is_a_crawler'} = 1;
-	$session->{'passive_session'} = 1;
+	$self->{'is_a_crawler'} = 1;
+	$self->{'passive_session'} = 1;
     }
-    $session->{'passive_session'} = 1 if ($rss||$action eq 'wsdl'||$action eq 'css');
+    $self->{'passive_session'} = 1 if ($rss||$action eq 'wsdl'||$action eq 'css');
 
     # if a session cookie exist, try to restore an existing session, don't store sessions from bots
-    if (($cookie)&&($session->{'passive_session'} != 1)){
+    if (($cookie)&&($self->{'passive_session'} != 1)){
 	my $status ;
-	$status = $session->load($cookie);
+	$status = $self->load($cookie);
 	unless (defined $status) {
 	    return undef;
 	}
@@ -81,57 +85,103 @@ sub new {
 	    do_log('info',"SympaSession::new ignoring unknown session cookie '$cookie'" ); # start a new session (may ne a fake cookie)
 	    return (new SympaSession ($robot));
 	}
-	# checking if the client host is unchanged during the session brake sessions when using multiple proxy with
-        # load balancing (round robin, etc). This check is removed until we introduce some other method
-	# if($session->{'remote_addr'} ne $ENV{'REMOTE_ADDR'}){
-	#    do_log('info','SympaSession::new ignoring session cookie because remote host %s is not the original host %s', $ENV{'REMOTE_ADDR'},$session->{'remote_addr'}); # start a new session
-	#    return (new SympaSession ($robot));
-	#}
     }else{
 	# create a new session context
-	$session->{'new_session'} = 1; ## Tag this session as new, ie no data in the DB exist
-        $session->{'id_session'} = &get_random();
-	$session->{'email'} = 'nobody';
-        $session->{'remote_addr'} = $ENV{'REMOTE_ADDR'};
-	$session->{'date'} = time;
-	$session->{'start_date'} = time;
-	$session->{'hit'} = 1;
-	$session->{'robot'} = $robot; 
-	$session->{'data'} = '';
+	$self->{'new_session'} = 1; ## Tag this session as new, ie no data in the DB exist
+	$self->{'id_session'} = &get_random();
+	$self->{'email'} = 'nobody';
+	$self->{'remote_addr'} = $ENV{'REMOTE_ADDR'};
+	$self->{'date'} = $self->{'refresh_date'} = $self->{'start_date'} =
+	    time;
+	$self->{'hit'} = 1;
+	$self->{'data'} = '';
     }
-    return $session;
+    return $self;
 }
 
 sub load {
     my $self = shift;
     my $cookie = shift;
-
-    do_log('debug', 'SympaSession::load(%s)', $cookie);
+    
+    Log::do_log('debug', '(%s)', $cookie);
 
     unless ($cookie) {
-	do_log('err', 'SympaSession::load() : internal error, SympaSession::load called with undef id_session');
+	do_log('err',
+	    'SympaSession::load() : internal error, SympaSession::load called with undef id_session');
 	return undef;
     }
-    
-    my $statement ;
-
-    $statement = "SELECT id_session AS id_session, date_session AS \"date\", remote_addr_session AS remote_addr, robot_session AS robot, email_session AS email, data_session AS data, hit_session AS hit, start_date_session AS start_date FROM session_table WHERE id_session = ?";
 
     my $dbh = &List::db_get_handler();
     my $sth;
+    my $statement;
+    my $id_session;
+    my $is_old_session = 0;
 
     ## Check database connection
     unless ($dbh and $dbh->ping) {
 	return undef unless &List::db_connect();
     }
-    unless ($sth = $dbh->prepare($statement)) {
-	do_log('err','Unable to prepare SQL statement : %s', $dbh->errstr);
-	return undef;
+
+    ## Load existing session.
+    if ($cookie and $cookie =~ /^\d{,16}$/) {
+	## Compatibility: session by older releases of Sympa.
+	$id_session = $cookie;
+	$is_old_session = 1;
+
+	## Session by older releases of Sympa doesn't have refresh_date.
+	$statement =
+	    q{SELECT id_session AS id_session, id_session AS prev_id,
+		     date_session AS "date",
+		     remote_addr_session AS remote_addr,
+		     email_session AS email,
+		     data_session AS data, hit_session AS hit,
+		     start_date_session AS start_date,
+		     date_session AS refresh_date
+	      FROM session_table
+	      WHERE id_session = ? AND
+		    refresh_date_session IS NULL};
+	unless ($sth = $dbh->prepare($statement)) {
+	    do_log('err', 'Unable to prepare SQL statement : %s',
+		$dbh->errstr);
+	    return undef;
+	}
+	unless ($sth->execute($id_session)) {
+	    do_log('err', 'Unable to execute SQL statement "%s" : %s',
+		$statement, $dbh->errstr);
+	    return undef;
+	}    
+    } else {
+	$id_session = decrypt_session_id($cookie);
+	unless ($id_session) {
+	    do_log('err',
+		'SympaSession::load() : internal error, SympaSession::load called with undef id_session');
+	    return 'not_found';
+	}
+
+	## Cookie may contain current or previous session ID.
+	$statement =
+	    q{SELECT id_session AS id_session, prev_id_session AS prev_id,
+		     date_session AS "date",
+		     remote_addr_session AS remote_addr,
+		     email_session AS email,
+		     data_session AS data, hit_session AS hit,
+		     start_date_session AS start_date,
+		     refresh_date_session AS refresh_date
+	      FROM session_table
+	      WHERE id_session = ? AND prev_id_session IS NOT NULL OR
+		    prev_id_session = ?};
+	unless ($sth = $dbh->prepare($statement)) {
+	    do_log('err', 'Unable to prepare SQL statement : %s',
+		$dbh->errstr);
+	    return undef;
+	}
+	unless ($sth->execute($id_session, $id_session)) {
+	    do_log('err', 'Unable to execute SQL statement "%s" : %s',
+		$statement, $dbh->errstr);
+	    return undef;
+	}    
     }
-    unless ($sth->execute($cookie)) {
-	do_log('err','Unable to execute SQL statement "%s" : %s', $statement, $dbh->errstr);
-	return undef;
-    }    
+
     my $session = undef;
     my $new_session = undef;
     my $counter = 0;
@@ -148,16 +198,28 @@ sub load {
     unless ($session) {
 	return 'not_found';
     }
-    
+
+    ## Compatibility: Upgrade session by older releases of Sympa.
+    if ($is_old_session) {
+	$sth = $dbh->prepare(
+	    q{UPDATE session_table
+	      SET prev_id_session = id_session
+	      WHERE id_session = ? AND prev_id_session IS NULL AND
+		    refresh_date_session IS NULL}
+	) and
+	$sth->execute($id_session);
+    }
+
     my %datas= &tools::string_2_hash($session->{'data'});
     foreach my $key (keys %datas) {$self->{$key} = $datas{$key};} 
 
     $self->{'id_session'} = $session->{'id_session'};
+    $self->{'prev_id'} = $session->{'prev_id'};
     $self->{'date'} = $session->{'date'};
+    $self->{'refresh_date'} = $session->{'refresh_date'};
     $self->{'start_date'} = $session->{'start_date'};
     $self->{'hit'} = $session->{'hit'} +1 ;
     $self->{'remote_addr'} = $session->{'remote_addr'};
-    $self->{'robot'} = $session->{'robot'};
     $self->{'email'} = $session->{'email'};    
 
     return ($self);
@@ -165,7 +227,6 @@ sub load {
 
 ## This method will both store the session information in the database
 sub store {
-
     my $self = shift;
     do_log('debug', '');
 
@@ -182,6 +243,7 @@ sub store {
     my $data_string = &tools::hash_2_string (\%hash);
     my $dbh = &List::db_get_handler();
     my $sth;
+    my $time = time;
 
     ## Check database connection
     unless ($dbh and $dbh->ping) {
@@ -192,19 +254,67 @@ sub store {
     if ($self->{'new_session'}) {
 
       ## Store the new session ID in the DB
-      my $add_statement = sprintf "INSERT INTO session_table (id_session, date_session, remote_addr_session, robot_session, email_session, start_date_session, hit_session, data_session) VALUES (%s,%d,%s,%s,%s,%d,%d,%s)",$dbh->quote($self->{'id_session'}),time,$dbh->quote($ENV{'REMOTE_ADDR'}),$dbh->quote($self->{'robot'}),$dbh->quote($self->{'email'}),$self->{'start_date'},$self->{'hit'}, $dbh->quote($data_string);
-      
+	## Previous session ID is set to be same as new session ID.
+	my $add_statement = sprintf
+	    q{INSERT INTO session_table
+	      (id_session, prev_id_session,
+	       date_session, refresh_date_session,
+	       remote_addr_session, robot_session,
+	       email_session, start_date_session, hit_session, data_session)
+	      VALUES (%s, %s, %d, %d, %s, %s, %s, %d, %d, %s)},
+	    $dbh->quote($self->{'id_session'}),
+	    $dbh->quote($self->{'id_session'}),
+	    $time, $time,
+	    $dbh->quote($ENV{'REMOTE_ADDR'}), $dbh->quote($self->{'robot'}),
+	    $dbh->quote($self->{'email'}), $self->{'start_date'},
+	    $self->{'hit'}, $dbh->quote($data_string);
       unless ($dbh->do($add_statement)) {
 	do_log('err','Unable to update session information in database while execute SQL statement "%s" : %s', $add_statement, $dbh->errstr);
 	return undef;
       }    
 
+	$self->{'prev_id'} = $self->{'id_session'};
+
       ## If the session already exists in DB, then perform an UPDATE
     }else {
-      
+	## Cookie may contain previous session ID.
+	my $statement = sprintf
+	    q{SELECT id_session
+	      FROM session_table
+	      WHERE prev_id_session = %s},
+	    $dbh->quote($self->{'id_session'});
+	my $sth = $dbh->prepare($statement);
+	unless ($sth) {
+	    do_log('err',
+		'Unable to update session information in database while execute SQL statement "%s" : %s',
+		$statement, $dbh->errstr
+	    );
+	    return undef;
+	}
+	$sth->execute;
+	if ($sth->rows) {
+	    my $new_id = $sth->fetchrow;
+	    $sth->finish;
+	    if ($new_id) {
+		$self->{'prev_id'} = $self->{'id_session'};
+		$self->{'id_session'} = $new_id;
+	    }
+	}
+
       ## Update the new session in the DB
-      my $update_statement = sprintf "UPDATE session_table SET date_session=%d, remote_addr_session=%s, robot_session=%s, email_session=%s, start_date_session=%d, hit_session=%d, data_session=%s WHERE (id_session=%s)",time,$dbh->quote($ENV{'REMOTE_ADDR'}),$dbh->quote($self->{'robot'}),$dbh->quote($self->{'email'}),$self->{'start_date'},$self->{'hit'}, $dbh->quote($data_string), $dbh->quote($self->{'id_session'});
-      
+      my $update_statement = sprintf
+	    q{UPDATE session_table
+	      SET date_session = %d, remote_addr_session = %s,
+		  robot_session = %s, email_session = %s,
+		  start_date_session = %d, hit_session = %d, data_session = %s
+	      WHERE id_session = %s AND prev_id_session IS NOT NULL OR
+		    prev_id_session = %s},
+	    $time, $dbh->quote($ENV{'REMOTE_ADDR'}),
+	    $dbh->quote($self->{'robot'}), $dbh->quote($self->{'email'}),
+	    $self->{'start_date'}, $self->{'hit'}, $dbh->quote($data_string),
+	    $dbh->quote($self->{'id_session'}),
+	    $dbh->quote($self->{'id_session'});
+
       unless ($dbh->do($update_statement)) {
 	do_log('err','Unable to update session information in database while execute SQL statement "%s" : %s', $update_statement, $dbh->errstr);
 	return undef;
@@ -216,7 +326,6 @@ sub store {
 
 ## This method will renew the session ID 
 sub renew {
-
     my $self = shift;
     do_log('debug', 'id_session=(%s)',$self->{'id_session'});
 
@@ -239,18 +348,107 @@ sub renew {
 	return undef unless &List::db_connect();
     }	   
 
+    ## Cookie may contain previous session ID.
+    my $statement = sprintf
+	q{SELECT id_session
+	  FROM session_table
+	  WHERE prev_id_session = %s},
+	$dbh->quote($self->{'id_session'});
+    $sth = $dbh->prepare($statement);
+    unless ($sth) {
+	do_log('err',
+	    'Unable to update session information in database while execute SQL statement "%s" : %s',
+	    $statement, $dbh->errstr
+	);
+	return undef;
+    }
+    $sth->execute;
+    if ($sth->rows) {
+	my $new_id = $sth->fetchrow;
+	$sth->finish;
+        if ($new_id) {
+            $self->{'prev_id'} = $self->{'id_session'};
+            $self->{'id_session'} = $new_id;
+        }
+    }
+
     ## Renew the session ID in order to prevent session hijacking
     my $new_id = &get_random();
 
-    ## First remove the DB entry for the previous session ID
-    my $update_statement = sprintf "UPDATE session_table SET id_session=%s WHERE (id_session=%s)",$dbh->quote($new_id), $dbh->quote($self->{'id_session'});
-    unless ($dbh->do($update_statement)) {
+    ## Do refresh the session ID when remote address was changed or refresh
+    ## interval was past.  Conditions also are checked by SQL so that
+    ## simultaneous processes will be prevented renewing cookie.
+    my $time = time;
+    my $remote_addr = $ENV{'REMOTE_ADDR'};
+    my $refresh_term;
+    if ($Conf::Conf{'cookie_refresh'} == 0) {
+	$refresh_term = $time;
+    } else {
+	my $cookie_refresh = $Conf::Conf{'cookie_refresh'};
+	$refresh_term =
+	    int($time - $cookie_refresh * 0.25 - rand($cookie_refresh * 0.5));
+    }
+    unless ($self->{'remote_addr'} ne $remote_addr or
+	$self->{'refresh_date'} <= $refresh_term) {
+	return 0;
+    }
+
+    ## First insert DB entry with new session ID,
+    my $add_statement = sprintf
+	q{INSERT INTO session_table
+	  (id_session, prev_id_session,
+	   start_date_session, date_session, refresh_date_session,
+	   remote_addr_session, robot_session, email_session,
+	   hit_session, data_session)
+	  SELECT %s, id_session,
+		 start_date_session, date_session, %d,
+		 %s, %s, email_session,
+		 hit_session, data_session
+	  FROM session_table
+	  WHERE (id_session = %s AND prev_id_session IS NOT NULL OR
+		 prev_id_session = %s) AND
+		(remote_addr_session <> %s OR refresh_date_session <= %d)},
+	$dbh->quote($new_id),
+	$time,
+	$dbh->quote($remote_addr), $dbh->quote($self->{'robot'}),
+	$dbh->quote($self->{'id_session'}),
+	$dbh->quote($self->{'id_session'}),
+	$dbh->quote($remote_addr), $refresh_term;
+    ## Keep previous ID to prevent crosstalk, clearing grand-parent ID.
+    my $update_statement = sprintf
+	q{UPDATE session_table
+	  SET prev_id_session = NULL
+	  WHERE id_session = %s},
+	$dbh->quote($self->{'id_session'});
+    ## Remove record of grand-parent ID.
+    my $del_statement = sprintf
+        q{DELETE FROM session_table
+          WHERE id_session = %s AND prev_id_session IS NULL},
+        $dbh->quote($self->{'prev_id'});
+
+    $sth = $dbh->prepare($add_statement);
+    unless ($sth) {
       do_log('err','Unable to renew session ID for session %s',$self->{'id_session'});
       return undef;
     }	 
+    $sth->execute;
+    unless ($sth->rows) {
+	return 0;
+    }
+    $dbh->do($update_statement);
+    $dbh->do($del_statement);
 
     ## Renew the session ID in order to prevent session hijacking
+    do_log('info',
+	'[robot %s] [session %s] [client %s]%s new session %s',
+	$self->{'robot'}, $self->{'id_session'}, $remote_addr,
+	($self->{'email'} ? sprintf(' [user %s]', $self->{'email'}) : ''),
+	$new_id
+    );
+    $self->{'prev_id'} = $self->{'id_session'};
     $self->{'id_session'} = $new_id;
+    $self->{'refresh_date'} = $time;
+    $self->{'remote_addr'} = $remote_addr;
 
     return 1;
 }
@@ -261,7 +459,7 @@ sub purge_old_sessions {
 
     my $robot = shift;
 
-    do_log('info', 'SympaSession::purge_old_sessions(%s,%s)',$robot);
+    do_log('info', 'SympaSession::purge_old_sessions(%s)',$robot);
 
     my $delay = &tools::duration_conv($Conf{'session_table_ttl'}) ; 
     my $anonymous_delay = &tools::duration_conv($Conf{'anonymous_session_table_ttl'}) ; 
@@ -282,12 +480,21 @@ sub purge_old_sessions {
     my $and = ' AND ' if (($delay_condition) && ($robot_condition));
     my $anonymous_and = ' AND ' if (($anonymous_delay_condition) && ($robot_condition));
 
-    my $count_statement = sprintf "SELECT count(*) FROM session_table WHERE $robot_condition $and $delay_condition";
-    my $anonymous_count_statement = sprintf "SELECT count(*) FROM session_table WHERE $robot_condition $anonymous_and $anonymous_delay_condition AND email_session = 'nobody' AND hit_session = '1'";
+    my $count_statement = sprintf
+	q{SELECT count(*) FROM session_table WHERE %s %s %s},
+	$robot_condition, $and, $delay_condition;
+    my $anonymous_count_statement = sprintf
+	q{SELECT count(*) FROM session_table
+	  WHERE %s %s %s AND email_session = 'nobody' AND hit_session = '1'},
+	$robot_condition, $anonymous_and, $anonymous_delay_condition;
 
-
-    my $statement = sprintf "DELETE FROM session_table WHERE $robot_condition $and $delay_condition";
-    my $anonymous_statement = sprintf "DELETE FROM session_table WHERE $robot_condition $anonymous_and $anonymous_delay_condition AND email_session = 'nobody' AND hit_session = '1'";
+    my $statement = sprintf
+	q{DELETE FROM session_table WHERE %s %s %s},
+	$robot_condition, $and, $delay_condition;
+    my $anonymous_statement = sprintf
+	q{DELETE FROM session_table
+	  WHERE %s %s %s AND email_session = 'nobody' AND hit_session = '1'},
+	$robot_condition, $anonymous_and, $anonymous_delay_condition;
 
     ## Check database connection
     unless ($dbh and $dbh->ping) {
@@ -346,7 +553,7 @@ sub purge_old_tickets {
 
     my $robot = shift;
 
-    do_log('info', 'SympaSession::purge_old_tickets(%s,%s)',$robot);
+    do_log('info', 'SympaSession::purge_old_tickets(%s)',$robot);
 
     my $delay = &tools::duration_conv($Conf{'one_time_ticket_table_ttl'}) ; 
 
@@ -410,11 +617,19 @@ sub list_sessions {
     my $and = ' AND ' if (($condition) && ($condition2));
     $condition = $condition.$and.$condition2 ;
 
-    my $condition3 =  " email_session != 'nobody' " if ($connected_only eq 'on');
+    my $condition3 =  " email_session <> 'nobody' " if ($connected_only eq 'on');
     my $and2 = ' AND '  if (($condition) && ($condition3));
     $condition = $condition.$and2.$condition3 ;
 
-    my $statement = sprintf "SELECT remote_addr_session, email_session, robot_session, date_session, start_date_session, hit_session FROM session_table WHERE $condition";
+    $condition .= sprintf '%sprev_id_session IS NOT NULL',
+	($condition ? ' AND ' : '');
+
+    my $statement = sprintf
+	q{SELECT remote_addr_session, email_session, robot_session,
+		 date_session, start_date_session, hit_session
+	  FROM session_table
+	  WHERE %s},
+	$condition;
     do_log('debug', 'SympaSession::list_session() : statement = %s',$statement);
 
     ## Check database connection
@@ -433,8 +648,8 @@ sub list_sessions {
     
     while (my $session = ($sth->fetchrow_hashref('NAME_lc'))) {
 
-	$session->{'formated_date'} = &Language::gettext_strftime ("%d %b %y  %H:%M", localtime($session->{'date_session'}));
-	$session->{'formated_start_date'} = &Language::gettext_strftime ("%d %b %y  %H:%M", localtime($session->{'start_date_session'}));
+	$session->{'formated_date'} = Language::gettext_strftime("%d %b %Y at %H:%M:%S", localtime($session->{'date_session'}));
+	$session->{'formated_start_date'} = Language::gettext_strftime("%d %b %Y at %H:%M:%S", localtime($session->{'start_date_session'}));
 
 	# do_log('debug', 'SympaSession::list_session() DUMP : %s,%s,%s,%s',$session->{'remote_addr_session'}, $session->{'email_session'}, $session->{'robot_session'}, $session->{'formated_date'});
 
@@ -484,10 +699,12 @@ sub set_cookie {
 	$http_domain="";
     }
 
+    my $value = encrypt_session_id($self->{'id_session'});
+
     my $cookie;
     if ($expires =~ /session/i) {
 	$cookie = new CGI::Cookie (-name    => 'sympa_session',
-				   -value   => $self->{'id_session'},
+				   -value   => $value,
 				   -domain  => $http_domain,
 				   -path    => '/',
 				   -secure => $use_ssl,
@@ -495,7 +712,7 @@ sub set_cookie {
 				   );
     }else {
 	$cookie = new CGI::Cookie (-name    => 'sympa_session',
-				   -value   => $self->{'id_session'},
+				   -value   => $value,
 				   -expires => $expiration,
 				   -domain  => $http_domain,
 				   -path    => '/',
@@ -508,13 +725,45 @@ sub set_cookie {
     printf "Set-Cookie: %s\n", $cookie->as_string;
     return 1;
 }
-    
+
+# Build an HTTP cookie value to be sent to a SOAP client
+sub soap_cookie2 {
+    my ($session_id, $http_domain, $expire) = @_;
+    my $cookie;
+    my $value;
+
+    # WARNING : to check the cookie the SOAP services does not gives
+    # all the cookie, only it's value so we need ':'
+    $value = encrypt_session_id($session_id);
+
+    ## With set-cookie2 max-age of 0 means removing the cookie
+    ## Maximum cookie lifetime is the session
+    $expire ||= 600; ## 10 minutes
+
+    if ($http_domain eq 'localhost') {
+	$cookie = CGI::Cookie->new(
+	    -name => 'sympa_session',
+	    -value => $value,
+	    -path => '/',
+	);
+	$cookie->max_age(time + $expire); # needs CGI >= 3.51.
+    } else {
+	$cookie = CGI::Cookie->new(
+	    -name => 'sympa_session',
+	    -value => $value,
+	    -domain => $http_domain,
+	    -path => '/',
+	);
+	$cookie->max_age(time + $expire); # needs CGI >= 3.51.
+    }
+
+    ## Return the cookie value
+    return $cookie->as_string;
+}
 
 sub get_random {
-    do_log('debug', 'SympaSession::random ');
-     my $random = int(rand(10**7)).int(rand(10**7)); ## Concatenates 2 integers for a better entropy
-     $random =~ s/^0(\.|\,)//;
-     return ($random)
+    ## Concatenates two integers for a better entropy.
+    return sprintf '%07d%07d', int(rand(10**7)), int(rand(10**7));
 }
 
 ## Return the session object content, as a hashref
@@ -529,5 +778,29 @@ sub as_hashref {
   return $data;
 }
 
-1;
+## Generate cookie from session ID.
+sub encrypt_session_id {
+    my $id_session = shift;
 
+    my $cipher = tools::ciphersaber_installed();
+    unless ($cipher) {
+	return "5e55$id_session";
+    }
+    return unpack 'H*', $cipher->encrypt(pack 'H*', $id_session);
+}
+
+## Get session ID from cookie.
+sub decrypt_session_id {
+    my $cookie = shift;
+
+    return undef unless $cookie and $cookie =~ /\A(?:[0-9a-f]{2})+\z/;
+
+    my $cipher = tools::ciphersaber_installed();
+    unless ($cipher) {
+	return undef unless $cookie =~ s/\A5e55//;
+	return $cookie;
+    }
+    return unpack 'H*', $cipher->decrypt(pack 'H*', $cookie);
+}
+
+1;
